@@ -35,7 +35,7 @@ extern void die(const char *,struct pt_regs *,long);
 
 static ATOMIC_NOTIFIER_HEAD(notify_page_fault_chain);
 
-int toi_faulted = 0;
+int toi_faulted;
 EXPORT_SYMBOL(toi_faulted);
 
 int register_page_fault_notifier(struct notifier_block *nb)
