@@ -26,17 +26,22 @@
 static int pr_index;
 
 #if 0
-#define PR_DEBUG(a, b...) do { if (pr_index < 20) printk(a, ##b); } while(0)
+#define PR_DEBUG(a, b...) do { \
+	if (pr_index < 20) \
+		printk(a, ##b); \
+} while (0)
 #else
-#define PR_DEBUG(a, b...) do { } while(0)
+#define PR_DEBUG(a, b...) do { } while (0)
 #endif
 
-#define MAX_OUTSTANDING_IO 16384
+#define TARGET_OUTSTANDING_IO 16384
 #define MAX_READAHEAD 2048
 #define CLEANUP_BATCH_SIZE 16
 
-static int max_outstanding_io = MAX_OUTSTANDING_IO;
-static int max_readahead = MAX_READAHEAD;
+static int target_outstanding_io = 2048;
+static atomic_t current_outstanding_io;
+static int max_outstanding_io;
+static int max_readahead = 2048;
 
 struct io_info {
 	struct bio *sys_struct;
@@ -51,6 +56,8 @@ static struct page *bio_queue_head, *bio_queue_tail;
 static DEFINE_SPINLOCK(bio_queue_lock);
 static atomic_t toi_io_queue_length;
 static int toi_io_max_queue_length;
+static int queue_trigger = 25;
+static int free_mem_throttle;
 
 static LIST_HEAD(ioinfo_ready_for_cleanup);
 static DEFINE_SPINLOCK(ioinfo_ready_lock);
@@ -64,14 +71,11 @@ static atomic_t toi_io_in_progress;
 static atomic_t toi_io_to_cleanup;
 static DECLARE_WAIT_QUEUE_HEAD(num_in_progress_wait);
 
-/* [Max] number of I/O operations pending */
-static atomic_t outstanding_io;
-
-static int extra_page_forward = 0;
+static int extra_page_forward;
 
 static unsigned long toi_readahead_flags[
 	DIV_ROUND_UP(MAX_READAHEAD, BITS_PER_LONG)];
-static spinlock_t toi_readahead_flags_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(toi_readahead_flags_lock);
 static struct page *toi_ra_pages[MAX_READAHEAD];
 static int readahead_index, ra_submit_index;
 
@@ -90,15 +94,20 @@ int toi_writer_buffer_posn;
 static struct toi_bdev_info *toi_devinfo;
 
 DEFINE_MUTEX(toi_bio_queue_mutex);
-
-#ifdef CONFIG_SMP
 DEFINE_MUTEX(toi_bio_mutex);
-#define TAKE_BIO_MUTEX(reason) mutex_lock(&toi_bio_mutex)
-#define DROP_BIO_MUTEX() mutex_unlock(&toi_bio_mutex)
-#else
-#define TAKE_BIO_MUTEX(reason) do { } while(0)
-#define DROP_BIO_MUTEX(reason) do { } while(0)
-#endif
+
+/**
+ * set_throttle: Set the point where we pause to avoid oom.
+ *
+ * Initially, this value is zero, but when we first fail to allocate memory,
+ * we set it (plus a buffer) and thereafter throttle i/o once that limit is
+ * reached.
+ */
+
+static void set_throttle(void)
+{
+	free_mem_throttle = nr_unallocated_buffer_pages() + 50;
+}
 
 /**
  * toi_bio_cleanup_one: Cleanup one bio.
@@ -142,6 +151,7 @@ static void toi_bio_cleanup_one(struct io_info *io_info)
 
 	toi_kfree(1, io_info);
 	atomic_dec(&toi_io_to_cleanup);
+	atomic_dec(&current_outstanding_io);
 }
 
 /**
@@ -181,7 +191,7 @@ static void toi_cleanup_completed_io(int all)
 	spin_unlock_irqrestore(&ioinfo_ready_lock, flags);
 }
 
-#define NUM_REASONS 8
+#define NUM_REASONS 9
 static atomic_t reasons[NUM_REASONS];
 static char *reason_name[NUM_REASONS] = {
 	"readahead not ready",
@@ -191,7 +201,8 @@ static char *reason_name[NUM_REASONS] = {
 	"synchronous I/O",
 	"bio mutex when reading",
 	"bio mutex when writing",
-	"toi_bio_queue_page_write"
+	"toi_bio_queue_page_write",
+	"memory low"
 };
 
 /**
@@ -224,7 +235,9 @@ static void do_bio_wait(int reason)
 	} else {
 		atomic_inc(&reasons[reason]);
 
-		io_schedule();
+		/* Wait for something to cleanup */
+		wait_event(num_in_progress_wait,
+				atomic_read(&toi_io_to_cleanup));
 		toi_cleanup_completed_io(0);
 	}
 }
@@ -274,7 +287,7 @@ static int toi_prepare_readahead(int index)
 
 	new_page = toi_get_zeroed_page(12, TOI_ATOMIC_GFP);
 
-	if(!new_page)
+	if (!new_page)
 		return -ENOMEM;
 
 	toi_ra_pages[index] = virt_to_page(new_page);
@@ -348,9 +361,12 @@ static int submit(struct io_info *io_info)
 	unsigned long flags;
 
 	while (!bio) {
-		bio = bio_alloc(TOI_ATOMIC_GFP,1);
-		if (!bio)
+		bio = bio_alloc(TOI_ATOMIC_GFP, 1);
+		if (!bio) {
+			set_throttle();
 			do_bio_wait(1);
+		}
+
 	}
 
 	bio->bi_bdev = io_info->dev;
@@ -360,7 +376,7 @@ static int submit(struct io_info *io_info)
 	io_info->sys_struct = bio;
 
 	if (bio_add_page(bio, io_info->bio_page, PAGE_SIZE, 0) < PAGE_SIZE) {
-		printk("ERROR: adding page to bio at %lld\n",
+		printk(KERN_INFO "ERROR: adding page to bio at %lld\n",
 				(unsigned long long) io_info->first_block);
 		bio_put(bio);
 		return -EFAULT;
@@ -392,26 +408,31 @@ static int submit(struct io_info *io_info)
 static struct io_info *get_io_info_struct(void)
 {
 	struct io_info *this = NULL;
+	int cur_outstanding_io;
+	int free_pages = nr_unallocated_buffer_pages();
 
-	if ((atomic_read(&toi_io_to_cleanup) +
-	       atomic_read(&toi_io_in_progress)) >= max_outstanding_io) {
-		wait_event(num_in_progress_wait,
-			atomic_read(&toi_io_in_progress) < max_outstanding_io);
-
-		toi_cleanup_completed_io(1);
+	/* Getting low on memory and I/O is in progress? */
+	while (unlikely(free_pages < free_mem_throttle) &&
+			atomic_read(&current_outstanding_io)) {
+		do_bio_wait(8);
+		free_pages = nr_unallocated_buffer_pages();
 	}
 
 	do {
-		this = toi_kmalloc(1, sizeof(struct io_info), TOI_ATOMIC_GFP);
+		this = toi_kzalloc(1, sizeof(struct io_info), TOI_ATOMIC_GFP);
 
 		if (this)
 			break;
 
+		set_throttle();
 		do_bio_wait(2);
 	} while (!this);
 
 	memset(this, 0, sizeof(struct io_info));
 	INIT_LIST_HEAD(&this->list);
+	cur_outstanding_io = atomic_add_return(1, &current_outstanding_io);
+	if (cur_outstanding_io > max_outstanding_io)
+		max_outstanding_io = cur_outstanding_io;	
 	return this;
 }
 
@@ -449,8 +470,10 @@ static void toi_do_io(int writing, struct block_device *bdev, long block0,
 	io_info->readahead_index = readahead_index;
 
 	if (io_info->readahead_index == -1) {
-		while (!(buffer_virt = toi_get_zeroed_page(13, TOI_ATOMIC_GFP)))
+		while (!(buffer_virt = toi_get_zeroed_page(13, TOI_ATOMIC_GFP))) {
+			set_throttle();
 			do_bio_wait(3);
+		}
 
 		io_info->bio_page = virt_to_page(buffer_virt);
 	} else {
@@ -470,9 +493,9 @@ static void toi_do_io(int writing, struct block_device *bdev, long block0,
 		waiting_on = io_info->bio_page;
 
 	/*
-	 * If writing, copy our data. The data is probably in lowmem, but we cannot be
-	 * certain. If there is no compression, we might be passed the actual source
-	 * page's address.
+	 * If writing, copy our data. The data is probably in lowmem, but we
+	 * cannot be certain. If there is no compression, we might be passed
+	 * the actual source page's address.
 	 */
 	if (writing) {
 		to = (char *) buffer_virt;
@@ -486,8 +509,6 @@ static void toi_do_io(int writing, struct block_device *bdev, long block0,
 
 	submit(io_info);
 
-	atomic_inc(&outstanding_io);
-
 	if (syncio)
 		do_bio_wait(4);
 }
@@ -513,15 +534,39 @@ static void toi_bdev_page_io(int writing, struct block_device *bdev,
 /**
  * toi_bio_memory_needed: Report amount of memory needed for block i/o.
  *
- * We want to have at least enough memory so as to have max_outstanding_io
- * transactions on the fly at once. If we can do more, fine.
+ * We want to have at least enough memory so as to have target_outstanding_io
+ * or more transactions on the fly at once. If we can do more, fine.
  */
 static int toi_bio_memory_needed(void)
 {
-	return (max_outstanding_io * (PAGE_SIZE + sizeof(struct request) +
+	return (max(target_outstanding_io, max_readahead) *
+			(PAGE_SIZE + sizeof(struct request) +
 				sizeof(struct bio) + sizeof(struct io_info)));
 }
 
+/*
+ * toi_bio_print_debug_stats
+ *
+ * Description:
+ */
+static int toi_bio_print_debug_stats(char *buffer, int size)
+{
+	int len = 0;
+
+	len = snprintf_used(buffer, size, "- Max readahead %d. Max "
+			"outstanding io %d.\n", max_readahead,
+			max_outstanding_io);
+
+	len += snprintf_used(buffer + len, size - len,
+		"  Memory_needed: %d x (%lu + %u + %u + %u) = %d bytes.\n",
+		max(target_outstanding_io, max_readahead),
+		PAGE_SIZE, (unsigned int) sizeof(struct request),
+		(unsigned int) sizeof(struct bio),
+		(unsigned int) sizeof(struct io_info), toi_bio_memory_needed());
+
+	return len;
+}
+
 /**
  * toi_set_devinfo: Set the bdev info used for i/o.
  *
@@ -546,16 +591,16 @@ static void dump_block_chains(void)
 	for (i = 0; i < toi_writer_posn.num_chains; i++) {
 		struct extent *this;
 
-		printk("Chain %d:", i);
-
 		this = (toi_writer_posn.chains + i)->first;
 
 		if (!this)
-			printk(" (Empty)");
+			continue;
+
+		printk(KERN_INFO "Chain %d:", i);
 
 		while (this) {
-			printk(" [%lu-%lu]%s", this->minimum, this->maximum,
-					this->next ? "," : "");
+			printk(" [%lu-%lu]%s", this->minimum,
+					this->maximum, this->next ? "," : "");
 			this = this->next;
 		}
 
@@ -563,8 +608,8 @@ static void dump_block_chains(void)
 	}
 
 	for (i = 0; i < 3; i++)
-		printk("Posn %d: Chain %d, extent %d, offset %lu.\n", i,
-				toi_writer_posn_save[i].chain_num,
+		printk(KERN_INFO "Posn %d: Chain %d, extent %d, offset %lu.\n",
+				i, toi_writer_posn_save[i].chain_num,
 				toi_writer_posn_save[i].extent_num,
 				toi_writer_posn_save[i].offset);
 }
@@ -576,7 +621,7 @@ static void dump_block_chains(void)
  * set at the start of reading the image header, to skip the first page
  * of the header, which is read without using the extent chains.
  */
-static int go_next_page(void)
+static int go_next_page(int writing)
 {
 	int i, max = (toi_writer_posn.current_chain == -1) ? 1 :
 	  toi_devinfo[toi_writer_posn.current_chain].blocks_per_page;
@@ -585,15 +630,18 @@ static int go_next_page(void)
 		toi_extent_state_next(&toi_writer_posn);
 
 	if (toi_extent_state_eof(&toi_writer_posn)) {
-		printk("Extent state eof. "
-			"Expected compression ratio too optimistic?\n");
-		dump_block_chains();
+		/* Don't complain if readahead falls off the end */
+		if (writing) {
+			printk(KERN_INFO "Extent state eof. "
+				"Expected compression ratio too optimistic?\n");
+			dump_block_chains();
+		}
 		return -ENODATA;
 	}
 
 	if (extra_page_forward) {
 		extra_page_forward = 0;
-		return go_next_page();
+		return go_next_page(writing);
 	}
 
 	return 0;
@@ -624,14 +672,17 @@ static int toi_bio_rw_page(int writing, struct page *page,
 {
 	struct toi_bdev_info *dev_info;
 
-	if (go_next_page()) {
-		printk("Failed to advance a page in the extent data.\n");
+	if (go_next_page(writing)) {
+		printk(KERN_INFO "Failed to advance a page in the extent "
+				"data.\n");
 		return -ENODATA;
 	}
 
 	if (current_stream == 0 && writing &&
-		toi_writer_posn.current_chain == toi_writer_posn_save[2].chain_num &&
-		toi_writer_posn.current_offset == toi_writer_posn_save[2].offset) {
+		toi_writer_posn.current_chain ==
+			toi_writer_posn_save[2].chain_num &&
+		toi_writer_posn.current_offset ==
+			toi_writer_posn_save[2].offset) {
 		dump_block_chains();
 		BUG();
 	}
@@ -678,7 +729,7 @@ static void toi_read_header_init(void)
 	readahead_index = ra_submit_index = -1;
 }
 
-static int toi_bio_queue_flush_pages(void);
+static int toi_bio_queue_flush_pages(int finish);
 static void toi_bio_queue_page_write(char **full_buffer);
 
 /**
@@ -690,9 +741,10 @@ static int toi_rw_cleanup(int writing)
 {
 	int i;
 
-	if (writing && toi_writer_buffer_posn) {
-		toi_bio_queue_page_write(&toi_writer_buffer);
-		toi_bio_queue_flush_pages();
+	if (writing) {
+		if (toi_writer_buffer_posn)
+			toi_bio_queue_page_write(&toi_writer_buffer);
+		toi_bio_queue_flush_pages(1);
 	}
 
 	if (writing && current_stream == 2)
@@ -702,7 +754,7 @@ static int toi_rw_cleanup(int writing)
 	toi_finish_all_io();
 
 	if (!writing)
-		for (i=0; i < max_readahead; i++)
+		for (i = 0; i < max_readahead; i++)
 			toi_cleanup_readahead(i);
 
 	current_stream = 0;
@@ -710,8 +762,8 @@ static int toi_rw_cleanup(int writing)
 	for (i = 0; i < NUM_REASONS; i++) {
 		if (!atomic_read(&reasons[i]))
 			continue;
-		printk("Waited for i/o due to %s %d times.\n", reason_name[i],
-				atomic_read(&reasons[i]));
+		printk(KERN_INFO "Waited for i/o due to %s %d times.\n",
+				reason_name[i], atomic_read(&reasons[i]));
 		atomic_set(&reasons[i], 0);
 	}
 	return 0;
@@ -748,7 +800,8 @@ static int toi_bio_read_page_with_readahead(void)
 	do {
 		if (toi_prepare_readahead(ra_submit_index)) {
 			/* We are supposed to have enough memory. */
-			printk("Failed to get readahead buffer page %d.\n", ra_submit_index);
+			printk(KERN_INFO "Failed to get readahead buffer page "
+					"%d.\n", ra_submit_index);
 			toi_alloc_print_debug_stats();
 			toi_message(TOI_ANY_SECTION, TOI_LOW, 1,
 				" - Free memory is %d.\n",
@@ -762,8 +815,14 @@ static int toi_bio_read_page_with_readahead(void)
 			ra_submit_index);
 
 		if (last_result) {
-			printk("Begin read chunk for page %d returned %d.\n",
-				ra_submit_index, last_result);
+			/* 
+			 * Don't complain about failing to do readahead past
+			 * the end of storage.
+			 */
+			if (last_result != -61)
+				printk(KERN_INFO "Begin read chunk for page %d "
+					"returned %d.\n",
+					ra_submit_index, last_result);
 			break;
 		}
 
@@ -772,7 +831,7 @@ static int toi_bio_read_page_with_readahead(void)
 		if (ra_submit_index == max_readahead)
 			ra_submit_index = 0;
 
-	} while((!last_result) && (ra_submit_index != readahead_index) &&
+	} while ((!last_result) && (ra_submit_index != readahead_index) &&
 			(!toi_readahead_ready(readahead_index)));
 
 wait:
@@ -793,11 +852,14 @@ wait:
  * toi_bio_queue_flush_pages
  */
 
-static int toi_bio_queue_flush_pages(void)
+static int toi_bio_queue_flush_pages(int finish)
 {
 	unsigned long flags;
 	int result = 0;
 
+	if (!finish && atomic_read(&toi_io_queue_length) < queue_trigger)
+		return 0;
+
 	if (!mutex_trylock(&toi_bio_queue_mutex))
 		return 0;
 
@@ -853,13 +915,11 @@ static void toi_bio_queue_page_write(char **full_buffer)
 
 	while (!*full_buffer) {
 		*full_buffer = (char *) toi_get_zeroed_page(11, TOI_ATOMIC_GFP);
-		if (!*full_buffer)
+		if (!*full_buffer) {
+			set_throttle();
 			do_bio_wait(7);
+		}
 	}
-
-	/* Don't let queue length get away on us */
-	while (atomic_read(&toi_io_queue_length) > 10)
-		yield();
 }
 
 /*
@@ -921,7 +981,7 @@ static int toi_bio_read_page(unsigned long *pfn, struct page *buffer_page,
 
 	pr_index++;
 
-	TAKE_BIO_MUTEX(5);
+	mutex_lock(&toi_bio_mutex);
 
 	if (toi_rw_buffer(READ, (char *) pfn, sizeof(unsigned long)) ||
 	    toi_rw_buffer(READ, (char *) buf_size, sizeof(int)) ||
@@ -931,7 +991,7 @@ static int toi_bio_read_page(unsigned long *pfn, struct page *buffer_page,
 	} else
 		PR_DEBUG("%d: PFN %ld, %d bytes.\n", pr_index, *pfn, *buf_size);
 
-	DROP_BIO_MUTEX();
+	mutex_unlock(&toi_bio_mutex);
 	kunmap(buffer_page);
 	return result;
 }
@@ -949,12 +1009,16 @@ static int toi_bio_read_page(unsigned long *pfn, struct page *buffer_page,
 static int toi_bio_write_page(unsigned long pfn, struct page *buffer_page,
 		unsigned int buf_size)
 {
-	char *buffer_virt = kmap(buffer_page);
+	char *buffer_virt;
 	int result = 0;
 
 	pr_index++;
 
-	TAKE_BIO_MUTEX(6);
+	if (unlikely(test_action_state(TOI_TEST_FILTER_SPEED)))
+		return 0;
+
+	mutex_lock(&toi_bio_mutex);
+	buffer_virt = kmap(buffer_page);
 
 	if (toi_rw_buffer(WRITE, (char *) &pfn, sizeof(unsigned long)) ||
 	    toi_rw_buffer(WRITE, (char *) &buf_size, sizeof(int)) ||
@@ -964,9 +1028,9 @@ static int toi_bio_write_page(unsigned long pfn, struct page *buffer_page,
 	PR_DEBUG("%d: Index %ld, %d bytes. Result %d.\n", pr_index, pfn,
 			buf_size, result);
 
-	DROP_BIO_MUTEX();
 	kunmap(buffer_page);
-	toi_bio_queue_flush_pages();
+	mutex_unlock(&toi_bio_mutex);
+	toi_bio_queue_flush_pages(0);
 	return result;
 }
 
@@ -1005,7 +1069,7 @@ static int toi_rw_header_chunk(int writing,
 
 	result = toi_rw_buffer(writing, buffer, buffer_size);
 	if (writing) {
-		int flush_result = toi_bio_queue_flush_pages();
+		int flush_result = toi_bio_queue_flush_pages(0);
 		if (!result)
 			result = flush_result;
 	}
@@ -1017,11 +1081,18 @@ static int toi_rw_header_chunk(int writing,
  */
 static int write_header_chunk_finish(void)
 {
-	if (!toi_writer_buffer_posn)
-		return 0;
+	int result = 0;
 
-	return toi_bio_rw_page(WRITE, virt_to_page(toi_writer_buffer),
-		-1) ? -EIO : 0;
+	toi_bio_queue_flush_pages(1);
+
+	if (toi_writer_buffer_posn) {
+		result = toi_bio_rw_page(WRITE,
+			virt_to_page(toi_writer_buffer), -1) ? -EIO : 0;
+	}
+
+	toi_finish_all_io();
+
+	return result;
 }
 
 /**
@@ -1040,7 +1111,7 @@ static int toi_bio_storage_needed(void)
 static int toi_bio_save_config_info(char *buf)
 {
 	int *ints = (int *) buf;
-	ints[0] = max_outstanding_io;
+	ints[0] = target_outstanding_io;
 	ints[1] = max_readahead;
 	return 2 * sizeof(int);
 }
@@ -1054,7 +1125,7 @@ static int toi_bio_save_config_info(char *buf)
 static void toi_bio_load_config_info(char *buf, int size)
 {
 	int *ints = (int *) buf;
-	max_outstanding_io  = ints[0];
+	target_outstanding_io  = ints[0];
 	max_readahead = ints[1];
 }
 
@@ -1068,6 +1139,9 @@ static int toi_bio_initialise(int starting_cycle)
 {
 	toi_writer_buffer = (char *) toi_get_zeroed_page(14, TOI_ATOMIC_GFP);
 
+	if (starting_cycle)
+		max_outstanding_io = 0;
+
 	return toi_writer_buffer ? 0 : -ENOMEM;
 }
 
@@ -1102,8 +1176,12 @@ struct toi_bio_ops toi_bio_ops = {
 };
 
 static struct toi_sysfs_data sysfs_params[] = {
-	{ TOI_ATTR("max_outstanding_io", SYSFS_RW),
-	  SYSFS_INT(&max_outstanding_io, 1, MAX_OUTSTANDING_IO, 0),
+	{ TOI_ATTR("target_outstanding_io", SYSFS_RW),
+	  SYSFS_INT(&target_outstanding_io, 0, TARGET_OUTSTANDING_IO, 0),
+	},
+
+	{ TOI_ATTR("queue_trigger", SYSFS_RW),
+	  SYSFS_INT(&queue_trigger, 1, 4096, 0),
 	},
 
 	{ TOI_ATTR("max_readahead", SYSFS_RW),
@@ -1111,12 +1189,12 @@ static struct toi_sysfs_data sysfs_params[] = {
 	},
 };
 
-static struct toi_module_ops toi_blockwriter_ops =
-{
+static struct toi_module_ops toi_blockwriter_ops = {
 	.name					= "lowlevel i/o",
 	.type					= MISC_HIDDEN_MODULE,
 	.directory				= "block_io",
 	.module					= THIS_MODULE,
+	.print_debug_info			= toi_bio_print_debug_stats,
 	.memory_needed				= toi_bio_memory_needed,
 	.storage_needed				= toi_bio_storage_needed,
 	.save_config_info			= toi_bio_save_config_info,
@@ -1125,7 +1203,8 @@ static struct toi_module_ops toi_blockwriter_ops =
 	.cleanup				= toi_bio_cleanup,
 
 	.sysfs_data		= sysfs_params,
-	.num_sysfs_entries	= sizeof(sysfs_params) / sizeof(struct toi_sysfs_data),
+	.num_sysfs_entries	= sizeof(sysfs_params) /
+		sizeof(struct toi_sysfs_data),
 };
 
 /**
